from django.contrib.auth.mixins import UserPassesTestMixin
from django.urls import reverse

from aircox.models import Episode, Program, StaticPage, Track
from aircox import forms
from ..filters import EpisodeFilters
from .page import PageListView
from .program import ProgramPageDetailView, BaseProgramMixin
from .page import PageUpdateView


__all__ = (
    "EpisodeDetailView",
    "EpisodeListView",
    "PodcastListView",
    "EpisodeUpdateView",
)


class EpisodeDetailView(ProgramPageDetailView):
    model = Episode

    def get_context_data(self, **kwargs):
        if "tracks" not in kwargs:
            kwargs["tracks"] = self.object.track_set.order_by("position")
        return super().get_context_data(**kwargs)

    def get_related_queryset(self):
        return (
            self.get_queryset().parent(self.object.parent).exclude(pk=self.object.pk).published().order_by("-pub_date")
        )

    def get_related_url(self):
        return reverse("episode-list", kwargs={"parent_slug": self.object.parent.slug})


class EpisodeListView(PageListView):
    model = Episode
    filterset_class = EpisodeFilters
    parent_model = Program
    attach_to_value = StaticPage.Target.EPISODES


class PodcastListView(EpisodeListView):
    attach_to_value = StaticPage.Target.PODCASTS
    queryset = Episode.objects.published().with_podcasts().order_by("-pub_date")


class EpisodeUpdateView(UserPassesTestMixin, BaseProgramMixin, PageUpdateView):
    model = Episode
    form_class = forms.EpisodeForm
    template_name = "aircox/episode_form.html"

    def test_func(self):
        program = self.get_object().program
        return self.request.user.has_perm("aircox.%s" % program.change_permission_codename)

    def get_success_url(self):
        return reverse("episode-detail", kwargs={"slug": self.get_object().slug})

    def get_tracklist_queryset(self, episode):
        return Track.objects.filter(episode=episode)

    def get_tracklist_formset(self, episode, **kwargs):
        kwargs.update(
            {
                "queryset": self.get_tracklist_queryset(episode),
                "initial": {
                    "episode": episode.id,
                },
            }
        )
        return forms.TrackFormSet(**kwargs)

    def get_playlist_queryset(self, episode):
        return episode.sound_set.all()

    def get_playlist_formset(self, episode, **kwargs):
        kwargs.update(
            {
                "queryset": self.get_playlist_queryset(episode),
                "initial": {
                    "program": episode.parent_id,
                    "episode": episode.id,
                },
            }
        )
        return forms.SoundFormSet(**kwargs)

    def get_context_data(self, **kwargs):
        kwargs.update(
            {
                "playlist_formset": self.get_playlist_formset(self.object),
                "tracklist_formset": self.get_tracklist_formset(self.object),
            }
        )
        return super().get_context_data(**kwargs)

    def post(self, request, *args, **kwargs):
        super().post(request, *args, **kwargs)
        formset = self.get_formset(request.POST)
        if formset.is_valid():
            formset.save()
            return super().form_valid(formset)
        else:
            return super().form_valid(formset)  # form_invalid(formset)
